#ifndef CONFIGURATION_H_INCLUDED
#define CONFIGURATION_H_INCLUDED

#include <qstring.h>
#include <qmap.h>
#include <qarray.h>
#include <qdir.h>

#include "kmdcodec.h"
#include "qscrobbler.h"

#define APPNAME "qscrobbler"
#define CONFIGPATHNAME ".qscrobbler"
#define CONFIGNAME "qscrobbler.conf"
#define CONFIGGROUPNAME "QScrobbler"
#define CONFIGVERSION 1

#define APPVERSION_MAJOR 0
#define APPVERSION_MINOR 6

#define BINNAME "qscrobbler"

class Configuration
{
public:
	static Configuration& singleton();
#define qConfig Configuration::singleton()

	Configuration();
	~Configuration();

	bool isValidConfig() const;
	void write() const;
	void read();
	
	QString getConfigPath() const
	{
		QString configPath = QDir::homeDirPath() + "/" + QString(CONFIGPATHNAME) + "/";
		QDir().mkdir(configPath);
		return configPath;
	}
	
	QCString getConfigName() const
	{
		return QCString(getConfigPath() + QString(CONFIGNAME)); 
	};
	

public:
	const QString username() const { return username_; }
	const QString md5Password() const { return md5Password_; }
	int cacheSaveInterval() const { return cacheSaveInterval_; }
	bool submissionPaused() const { return submissionPaused_; }
	
	void setUsername(const QString &username) { username_ = username; }
	void setMD5Password(const QString &password) { md5Password_ = password; }
	void setPassword(const QString &password) 
	{
	    KMD5 m(password.utf8());
	    md5Password_ = m.hexDigest();
	}

	void setCacheSaveInterval(int interval) { cacheSaveInterval_ = interval; }
	void setSubmissionPaused(bool paused) { submissionPaused_ = paused; }
	
private:
	static const int configVersion = CONFIGVERSION;
	
	QString username_;
	QString md5Password_;
	int cacheSaveInterval_;
	bool submissionPaused_;
};

void saveWidgetPosition(QWidget *widget, const QString &configName, const QString &groupName);
void loadWidgetPosition(QWidget *widget, const QString &configName, const QString &groupName, int defX = 10, int defY = 10, int defWidth = 620, int defHeight = 400);

#endif // CONFIGURATION_H_INCLUDED
