#!/bin/bash

DISTRO_ROOT="/home/darkstar/Playground/zaurus/qscrobbler/distro"
INPUT_ROOT="$DISTRO_ROOT/input"
OUTPUT_ROOT="$DISTRO_ROOT/output"

DEV_ROOT="/home/darkstar/Playground/zaurus"
QUASAR_ROOT="$DEV_ROOT/qscrobbler"

function compileForSharp()
{
	cd "$DEV_ROOT"
	source dev-arm-qpe.sh

	cd "$QUASAR_ROOT"
	make clean > /dev/null 2>&1
	tmake qscrobbler-qtopia.pro > Makefile
	make clean > /dev/null 2>&1
	make -j3 > /dev/null 2>&1

	if [ ! -f "$QUASAR_ROOT/qscrobbler" ]
	then
		echo "Build failed..."
		exit 1
	else
		echo "Build successful."
	fi
	
	arm-linux-strip --strip-all "$QUASAR_ROOT/qscrobbler"
	
	rm -Rf "$INPUT_ROOT"
	mkdir -p "$INPUT_ROOT"
	cp -R "$DISTRO_ROOT/skeletons/sharp/"* "$INPUT_ROOT"
	cp "$QUASAR_ROOT/qscrobbler" "$INPUT_ROOT/opt/QtPalmtop/bin/qscrobbler"
	make clean > /dev/null 2>&1

	# copy images
	IMAGE_ROOT="$INPUT_ROOT/opt/QtPalmtop/pics"
	rm -f "$IMAGE_ROOT/"* > /dev/null 2>&1
	cp "$DISTRO_ROOT/images/"* "$IMAGE_ROOT/"
	
	# create package
	cd "$DISTRO_ROOT"
	find "$INPUT_ROOT" -name ".svn" -exec /bin/rm -rf '{}' \; > /dev/null 2>&1
	./ipkg-build "$INPUT_ROOT" "$OUTPUT_ROOT"
}

function compileForPdaXrom()
{
	cd "$DEV_ROOT"
	source dev-arm-pdaxrom.sh

	cd "$QUASAR_ROOT"
	make clean > /dev/null 2>&1
	qmake qscrobbler-pdaxrom.pro > Makefile
	make clean > /dev/null 2>&1
	make -j3 > /dev/null 2>&1

	if [ ! -f "$QUASAR_ROOT/qscrobbler" ]
	then
		echo "Build failed..."
		exit 1
	else
		echo "Build successful."
	fi

	armv5tel-cacko-linux-strip -s "$QUASAR_ROOT/qscrobbler"

	rm -Rf "$INPUT_ROOT"
	mkdir -p "$INPUT_ROOT"
	cp -R "$DISTRO_ROOT/skeletons/pdaXrom/"* "$INPUT_ROOT"
	cp "$QUASAR_ROOT/qscrobbler" "$INPUT_ROOT/usr/bin/qscrobbler"
	make clean > /dev/null 2>&1
	
	# copy images
	SHARE_ROOT="$INPUT_ROOT/usr/share/qscrobbler"
	IMAGE_ROOT="$SHARE_ROOT/pics"
	rm -f "$IMAGE_ROOT/"* > /dev/null 2>&1
	cp "$DISTRO_ROOT/images/"* "$IMAGE_ROOT/"
	
	# create package
	cd "$DISTRO_ROOT"
	find "$INPUT_ROOT" -name ".svn" -exec /bin/rm -rf '{}' \; > /dev/null 2>&1
	./ipkg-build "$INPUT_ROOT" "$OUTPUT_ROOT"
}

rm -Rf "$INPUT_ROOT"

mkdir -p "$INPUT_ROOT"
mkdir -p "$OUTPUT_ROOT"

echo
echo "Building for Qtopia..."
compileForSharp

echo
echo "Building for pdaXrom..."
compileForPdaXrom

echo

rm -Rf "$INPUT_ROOT"
