/*
 * Copyright (C) 2007 Andre Beckedorf <evilJazz _AT_ katastrophos _DOT_ net>
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <time.h>

#ifdef QTOPIA
#include <qpe/qpemenubar.h>
#include <qpe/qpetoolbar.h>
#include <qpe/qcopenvelope_qws.h>
#include <qpe/resource.h>
#include <qpe/config.h>
#include <qpe/global.h>
#else
#include <qmenubar.h>
#include <qtoolbar.h>
#include <resource.h>
#include <config.h>
// for Qtopia we are using the QPE variants for toolbar and menubar 
// to enable special features like the menu key etc.
// Define aliases to avoid ifdefs...
#define QPEMenuBar QMenuBar
#define QPEToolBar QToolBar
#endif

#include <qstringlist.h>
#include <qstring.h>
#include <qpushbutton.h>
#include <qlayout.h>
#include <qwidget.h>
#include <qlabel.h>
#include <qlistbox.h>
#include <qlistview.h>
#include <qspinbox.h>
#include <qlineedit.h>
#include <qcheckbox.h>
#include <qcombobox.h>
#include <qvgroupbox.h>
#include <qvbox.h>
#include <qhbox.h>
#include <qprogressbar.h>
#include <qtabbar.h>
#include <qdatastream.h>
#include <qwidgetstack.h>
#include <qmessagebox.h>

#include <qurlinfo.h>
#include <qfile.h>
#include <qregexp.h>

#include "qscrobbler.h"
#include "configuration.h"
#include "configurationdialog.h"
#include "debug.h"
#include "audioscrobbler.h"
#include "kascachelist.h"

#include "kmdcodec.h"

QScrobbler::QScrobbler()
{
	qConfig.read();
	setCaption("QScrobbler");
	
	QVBoxLayout *vl = new QVBoxLayout(this, 8);
	vl->setAutoAdd(true);

    scrobbler = new AudioScrobbler(this);
	scrobbler->setSubmissionPaused(qConfig.submissionPaused());
	
    updateScrobblerSettings();    
    
    QPopupMenu *tools = new QPopupMenu(this);
    tools->insertItem(tr("&Settings"), this, SLOT(showSettings()));
    
    QPopupMenu *help = new QPopupMenu(this);
    help->insertItem(tr("&About..."), this, SLOT(about()));
    
    QPEMenuBar *menu = new QPEMenuBar(this);
    menu->insertItem("&Tools", tools);
    menu->insertItem("&Help", help);
    menu->setSeparator( QMenuBar::InWindowsStyle );
    
    gbox1 = new QVGroupBox(tr("Song waiting to be scrobbled"), this);
    currentlyPlayingLabel = new QLabel(tr("No song."), gbox1);
    
    progressStack = new QWidgetStack(gbox1);
    progressStack->setSizePolicy(QSizePolicy(QSizePolicy::Expanding, QSizePolicy::Expanding));    
    progressBar = new QProgressBar(progressStack);
    progressStatus = new QLabel(progressStack);
    
    
    gbox0 = new QVGroupBox(tr("Scrobbled songs waiting to be submitted"), this);
    scrobbledTracksListView = new QListView(gbox0);
    scrobbledTracksListView->setItemMargin(4);
    scrobbledTracksListView->setAllColumnsShowFocus(true);
    scrobbledTracksListView->setSorting(-1);
    scrobbledTracksListView->setShowSortIndicator(false);    
    
    scrobbledTracksListView->addColumn("Played at (UTC)");
    scrobbledTracksListView->addColumn("Title");
    scrobbledTracksListView->addColumn("Artist");
    scrobbledTracksListView->addColumn("Album");
    
    QFont font = scrobbledTracksListView->font();
    font.setPointSize(font.pointSize() * 3 / 4);
    scrobbledTracksListView->setFont(font);
    
    QHBox *hbox = new QHBox(gbox0);
    QPushButton *pauseSubmissionsButton = new QPushButton("Pause submissions", hbox);
    pauseSubmissionsButton->setToggleButton(true);
    pauseSubmissionsButton->setOn(qConfig.submissionPaused());
    connect(pauseSubmissionsButton, SIGNAL(toggled(bool)), scrobbler, SLOT(setSubmissionPaused(bool)));

    QPushButton *clearSubmissionsButton = new QPushButton("Clear list", hbox);
    connect(clearSubmissionsButton, SIGNAL(clicked()), this, SLOT(clearScrobbledTracksList()));
    
    QVGroupBox *gbox2 = new QVGroupBox(tr("Status:"), this);
    hbox = new QHBox(gbox2);
    QLabel *statusLabel = new QLabel("ready.", hbox, "statuslabel2");
    
    QWidget *badge = new QWidget(hbox);
    badge->setBackgroundPixmap(Resource::loadPixmap("lastfm"));
    badge->setFixedSize(badge->backgroundPixmap()->width(), badge->backgroundPixmap()->height());
    
    connect(scrobbler, SIGNAL(statusMessage(const QString&)), statusLabel, SLOT(setText(const QString&)));
    connect(scrobbler, SIGNAL(cacheSaved()), this, SLOT(cacheSaved()));
    connect(scrobbler, SIGNAL(songAddedForSubmission(const QString&)), this, SLOT(songAddedForSubmission(const QString&)));

#ifdef QTOPIA
    QCopChannel *channel = new QCopChannel("QPE/MediaPlayer", this);
    connect(channel, SIGNAL(received(const QCString&, const QByteArray&)),
    		this, SLOT(mediaPlayerMessage(const QCString&, const QByteArray&)));
#else
    filePollTimer = new QTimer();
    connect(filePollTimer, SIGNAL(timeout()), this, SLOT(filePollTimerTriggered()));
    filePollTimer->start(1000, false);    
#endif    
    
    updateTimer = new QTimer();
    connect(updateTimer, SIGNAL(timeout()), this, SLOT(updateTimerTriggered()));
	
    time.start();
    pauseTime.start();
    pauseTimeDelta = 0;
    	
    cacheSaved();
}

QScrobbler::~QScrobbler()
{
}

void QScrobbler::closeEvent(QCloseEvent *e)
{
	qConfig.setSubmissionPaused(scrobbler->submissionPaused());
	qConfig.write();
	
	e->accept();
}

void QScrobbler::cacheSaved()
{
	DENTERMETHOD("QScrobbler::cacheSaved()");
	
	int subCount = scrobbler->cache().submissionCount();

    scrobbledTracksListView->setUpdatesEnabled(false);
    scrobbledTracksListView->blockSignals(true);
    scrobbledTracksListView->clear();
    
    for (int i = 0; i < subCount; i++)
    {
    	const ScrobSongData songData = scrobbler->cache().getSubmission(i);
    	QListViewItem *li = new QListViewItem(scrobbledTracksListView);
    	li->setText(0, songData.time);
    	li->setText(1, songData.songtitle);
    	li->setText(2, songData.artist);
    	li->setText(3, songData.album);
    }
    
    if (scrobbledTracksListView->firstChild())
    {
    	scrobbledTracksListView->setCurrentItem(scrobbledTracksListView->firstChild());
    	scrobbledTracksListView->setSelected(scrobbledTracksListView->firstChild(), true);
    }
    
    scrobbledTracksListView->blockSignals(false);
    scrobbledTracksListView->setUpdatesEnabled(true);
    scrobbledTracksListView->triggerUpdate();
    
    gbox0->setTitle(tr("Scrobbled songs waiting to be submitted") + ": " + QString::number(subCount));
    
    DEXITMETHOD("QScrobbler::cacheSaved()");
}

void QScrobbler::clearScrobbledTracksList()
{
	int ret = QMessageBox::warning(this, tr("Confirmation"),
								tr("Do you really want to clear all scrobbled tracks?"),
								tr("Yes"), tr("No"), QString::null,
								1, 1);

	if (ret == 0)
		scrobbler->clearCache();
}

void QScrobbler::songAddedForSubmission(const QString& songTitle)
{
	DENTERMETHOD("QScrobbler::songAddedForSubmission(...)");
	
	gbox1->setTitle(tr("Song currently playing"));
	progressStatus->setText(tr("Song successfully scrobbled."));
	
	progressStack->raiseWidget(progressStatus);
	progressBar->setProgress(100);
	updateTimer->stop();
	
	DEXITMETHOD("QScrobbler::songAddedForSubmission(...)");
}

void QScrobbler::showSettings()
{
	ConfigurationDialog configDialog(qConfig, this, "Configuration Dialog");

	if (configDialog.exec() == QDialog::Accepted)
	{
		qConfig = configDialog.result();
		qConfig.write();
		updateScrobblerSettings();
	}	
}

void QScrobbler::updateScrobblerSettings()
{
    scrobbler->setUsername(qConfig.username());
    scrobbler->setMD5Password(qConfig.md5Password());
    scrobbler->setCacheSaveInterval(qConfig.cacheSaveInterval());
    scrobbler->setCacheFileName(qConfig.getConfigPath() + "qscrobbler.cache");
    scrobbler->reinitialize();
}

void QScrobbler::about()
{
	QMessageBox mb("About QScrobbler", 
			"QScrobbler " + QString("%1.%2").arg(APPVERSION_MAJOR).arg(APPVERSION_MINOR) + "\n"
			"Copyright (C) 2007-2008 by Andre Beckedorf\n"
			"evilJazz@katastrophos.net\n"
			"http://katastrophos.net/qscrobbler\n\n"
			"QScrobbler is based on portions of KAudioScrobbler\n"
			"Copyright (C) 2004 by Steven Scott\n"
			"http://code.google.com/p/kaudioscrobbler/\n\n"
			"This software is licensed under the terms of\n"
			"the GNU Public License version 2.", QMessageBox::NoIcon, 1, 0, 0);
	mb.exec(); 
}

void QScrobbler::updateTimerTriggered()
{
	DENTERMETHOD("QScrobbler::updateTimerTriggered()");
	
	int secsElapsed = (time.elapsed() - pauseTimeDelta) / 1000;
	
	if (secsElapsed > progressBar->totalSteps())
	{
		progressBar->setProgress(100);
		updateTimer->stop();
	}
	else		
		progressBar->setProgress(secsElapsed);
		
	DEXITMETHOD("QScrobbler::updateTimerTriggered()");
}

void QScrobbler::mediaPlayerMessage(const QCString &msg, const QByteArray &data)
{
#ifdef QTOPIA
	DPRINTF("mediaPlayerMessage: %s", (const char*)msg);
	
	QDataStream stream(data, IO_ReadOnly);
	
	QString message = QString(msg);
	QString messageType = message.left(message.find('('));
	
	QString params = message.mid(message.find('(') + 1);
	params = params.left(params.find(')'));
	
	//DPRINTF("params: %s", (const char*)params.utf8());
	
	QStringList paramsParts = QStringList::split(',', params); 
	
	//DPRINTF("messageType: %s", (const char*)messageType.utf8());
	//DPRINTF("paramsParts: %s", (const char*)paramsParts.join(" ").utf8());
	
	if (messageType == "playingAudio")
	{
		DPRINTF("playing...");
		if (paramsParts.count() == 5)
		{
			QString filename;
			QString title;
			QString artist;
			QString album;
			int length;
						
			if (paramsParts[0].stripWhiteSpace() == "QString" &&
				paramsParts[1].stripWhiteSpace() == "QString" &&
				paramsParts[2].stripWhiteSpace() == "QString" &&
				paramsParts[3].stripWhiteSpace() == "QString" &&
				paramsParts[4].stripWhiteSpace() == "int")
			{
				stream >> filename >> title >> artist >> album;
				stream >> length;
			}
			else
				return;

			DPRINTF("filename: %s", (const char*)filename.utf8());
			DPRINTF("title: %s", (const char*)title.utf8());
			DPRINTF("artist: %s", (const char*)artist.utf8());
			DPRINTF("album: %s", (const char*)album.utf8());
			DPRINTF("length: %d", length);
			
			setCurrentSong(title, artist, album, length);
		}
	}
	else if (messageType == "paused")
	{
		DPRINTF("paused...");
		setCurrentSongPaused();
	}
	else if (messageType == "unpaused")
	{
		DPRINTF("playing again...");
		setCurrentSongUnpaused();
	}
	else if (messageType == "stopped")
	{
		removeCurrentSong();
	}
	else if (messageType == "seeked")
	{
		DPRINTF("seeked...");
		scrobbler->seek();
	}
#endif
}

void QScrobbler::setCurrentSong(const QString &title, const QString &artist, const QString &album, int length)
{
	DENTERMETHOD("QScrobbler::setCurrentSong(...)");
	
	int remainingTime = scrobbler->play(artist, title, album, "", length);
	time.restart();
	pauseTimeDelta = 0;
	
	gbox1->setTitle(tr("Song waiting to be scrobbled"));
	currentlyPlayingLabel->setText(artist + " - " + title);
	progressBar->setTotalSteps(remainingTime);
	progressStack->raiseWidget(progressBar);
	progressBar->setProgress(0);
	updateTimer->start(3000);
	
	DEXITMETHOD("QScrobbler::setCurrentSong(...)");
}

void QScrobbler::removeCurrentSong()
{
	DPRINTF("stopped...");
	gbox1->setTitle(tr("Song waiting to be scrobbled"));
	currentlyPlayingLabel->setText(tr("No song."));
	progressStatus->setText("");
	progressStack->raiseWidget(progressStatus);
	
	scrobbler->stop();
	updateTimer->stop();
}

void QScrobbler::setCurrentSongPaused()
{
	updateTimer->stop();
	scrobbler->pause();
	pauseTime.restart();
}

void QScrobbler::setCurrentSongUnpaused()
{
	updateTimer->start(3000);
	scrobbler->unpause();
	pauseTimeDelta += pauseTime.elapsed();
}

#ifndef QTOPIA
QString QScrobbler::getTemporaryPlaybackStatusFileName()
{
	QString tempDir(::getenv("TMP"));
	if (tempDir.isEmpty())
	{
		tempDir = QString(::getenv("TMP"));
		if (tempDir.isEmpty())
		{
			tempDir = "/tmp";
		}
	}
	
	QString username(::getenv("USER"));
	if (!username.isEmpty())
		username = "_" + username;
	
	return tempDir + "/media" + username + ".sh";
}
#endif

QString QScrobbler::deescapeString(const QString &in)
{
	QString retval = in;
	
	// remove quotation marks...
	if (retval.startsWith("\"") && retval.findRev('"') == retval.length() - 1)
	{
		retval = in.mid(1, retval.length() - 2);
	}
	
	retval.replace(QRegExp("\\\""), "\"");
	retval.replace(QRegExp("\\\\"), "\\");
	retval.replace(QRegExp("\\$"), "$");
	return retval;
}

void QScrobbler::filePollTimerTriggered()
{
#ifndef QTOPIA
	QFile file(getTemporaryPlaybackStatusFileName());
	
	if (file.exists())
	{
		QDateTime currentModifiedDate = QFileInfo(file).lastModified();
		
		if ((lastDateTimeOfPolledFile < currentModifiedDate) && 
			(currentModifiedDate > QDateTime::currentDateTime().addSecs(-5)))
		{
			file.open(IO_ReadOnly);
		
			lastDateTimeOfPolledFile = currentModifiedDate;
			
			QTextStream ts(&file);
			ts.setEncoding(QTextStream::UnicodeUTF8);
			
			QString status = "";
			QString mediatype = "";
			QString filename = "";
			QString title = "";
			QString artist = "";
			QString album = "";
			int length = 0;
			
			while (!ts.eof())
			{
				QString line = ts.readLine();
			
				int sepIndex = line.find('=');
	
				if (!line.startsWith("#") && !line.stripWhiteSpace().isEmpty() && sepIndex > 0)
				{
					QString key = line.left(sepIndex);
					QString value = deescapeString(line.mid(sepIndex + 1));
					
					if (key == "status")
						status = value;
					else if (key == "mediatype")
						mediatype = value;
					else if (key == "artist")
						artist = value;
					else if (key == "album")
						album = value;
					else if (key == "title")
						title = value;
					else if (key == "length")
						length = value.toInt();
				}
			}
			
			QString currentStatus = status.lower();
			
			if (currentStatus == "playback_started" && length > 0)
				setCurrentSong(title, artist, album, length);
			else if (currentStatus == "playing")
				setCurrentSongUnpaused();
			else if (currentStatus == "paused")	
				setCurrentSongPaused();
			else if (currentStatus == "stopped")
				removeCurrentSong();
			
			file.close();
		}
	}	
#endif
}
